use std::{collections::HashMap, time::Duration};

use color_eyre::eyre::{Result, WrapErr};
use crossterm::event::{KeyCode, KeyEvent};
use ratatui::{prelude::*, widgets::*};
use serde::{Deserialize, Serialize};
use tokio::sync::mpsc::UnboundedSender;
use tokio::sync::OnceCell;

use super::{Buflist, Component};
use crate::{
  action::Action,
  config::{Config, KeyBindings},
};

#[derive(Default)]
pub struct Home {
  command_tx: Option<UnboundedSender<Action>>,
  config: OnceCell<Config>,
  buflist: Buflist,
}

impl Home {
  pub fn new() -> Self {
    Self::default()
  }
}

impl Component for Home {
  fn register_action_handler(&mut self, tx: UnboundedSender<Action>) -> Result<()> {
    self.buflist.register_action_handler(tx.clone())?;
    self.command_tx = Some(tx);
    Ok(())
  }

  fn register_config_handler(&mut self, config: Config) -> Result<()> {
    self.buflist.register_config_handler(config.clone())?;
    self
      .config
      .set(config)
      .context("Home config was already set")?;
    Ok(())
  }

  fn update(&mut self, action: &Action) -> Result<Option<Action>> {
    self.buflist.update(action)?;
    match action {
      Action::Tick => {},
      _ => {},
    }
    Ok(None)
  }

  fn draw(
    &mut self,
    frame: &mut Frame<'_>,
    area: Rect,
    buffers: &crate::buffers::Buffers,
  ) -> Result<()> {
    let layout = Layout::default()
      .direction(Direction::Horizontal)
      .constraints(vec![Constraint::Percentage(20), Constraint::Percentage(80)])
      .split(area);

    self
      .buflist
      .draw(frame, layout[0], buffers)
      .context("Error drawing buflist")?;
    frame.render_widget(
      Paragraph::new(format!(
        "content of {} goes here",
        buffers.active_buffer().short_name()
      ))
      .block(Block::new().borders(Borders::ALL)),
      layout[1],
    );
    Ok(())
  }
}
